/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only.   Notwithstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file Append.h
 * Header file for the definitions of the Append helper methods.
 */

///////////////////////////////////////////////////////////////////////////////
// pragmas
///////////////////////////////////////////////////////////////////////////////
#pragma once


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////
#include <string>
#include <vector>
#include <stdlib.h>

/**
*
* This file contains:
* Declarations for commonly needed Append functions
* Notes for using other routines for various types (e.g., string)
* Notes for *Reading* in various types
*/


/**
 * This will append an unsigned int value to the passed in vector data
 *
 * @param vecData - The vector to append the data to.
 * @param nValue   - The value to append
 */
void Append( std::vector<unsigned char>& vecData, unsigned int nValue );

/**
 * This will append an unsigned short value to the passed in vector data
 *
 * @param vecData - The vector to append the data to.
 * @param nValue   - The value to append
 */
void Append( std::vector<unsigned char>& vecData, unsigned short nValue );

/**
 * This will append an unsigned character buffer to the passed in vector data
 *
 * @param vecData - The vector to append the data to.
 * @param pcValue - The data to append.
 * @param nValueLength - The length of the data to append
 */
void Append( std::vector<unsigned char>& vecData, const unsigned char* pcValue, size_t nValueLength );

/**
 * This will append a char buffer to the passed in vector data
 *
 * @param vecData - The vector to append the data to.
 * @param pcValue - The data to append.
 * @param nValueLength - The length of the data to append
 */
void Append( std::vector<unsigned char>& vecData, const char* pcValue, size_t nValueLength );

/**
 * This will append a std::string to the passed in vector data
 *
 * @param vecData - The vector to append the data to.
 * @param strValue - The data to append.
 * @param nFieldSize - The length of the data to append
 * @param cFill - A character that will fill any leftover space
 */
void AppendFixedLen( std::vector<unsigned char>& vecData, const std::string& strValue, size_t nFieldSize, char cFill = '\0' );

/**
 * This will append a vector of bits to the passed in vector data
 *
 * @param vecOut - The vector to append the data to.
 * @param vecBitsIn - The vector from which the data will be used for the append operation.
 */
void Append( std::vector<unsigned char>& vecOut, const std::vector<bool>& vecBitsIn );

//from strings use:
//vecData.insert( vecData.end(), strData.begin(), strData.end());

//from a (unsigned) char* use:
//vecData.insert( vecData.end(), pcData, pcData + nDataLength );

//from another vector<(unsigned) char> use:
//vecData.insert( vecData.end(), vecBinField.begin(), vecBinField.end());


//Notes on reading FROM a vector into various types

//into integers use:
//int nLhs = CConvert::Swap32( *(int*)&vecData[13]);		//make sure vecData is long enough
//short nLhs = CConvert::Swap16( *(short*)&vecData[13]);

//into char* use:
//memcpy( pcDest, &vecData[13], nDataLength );  //make sure vecData && pcDest is long enough

//into strings use:
//strData.assign( &vecData[13], &vecData[13] + nDataLength );
//
//into wstring, convert to string first
