/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file CrtSecureFunctions.h
 * Header file which contains definitions for secure versions of standard C
 * functions.
 */


//#pragma once intentionally left out

//Defines secure versions of standard C functions.
//Wrapped by a test to see if they're already defined.  Right now the test uses a 
//macro that Microsoft defines.  Replace w/ more portable test when available.
#ifndef __GOT_SECURE_LIB__
//#define __GOT_SECURE_LIB__ intentionally left out


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////

#include <stdlib.h> //for size_t
#include <stdio.h>
#include <time.h>


///////////////////////////////////////////////////////////////////////////////
// Defines
///////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
extern "C" { 
#endif


#ifndef _countof
#define _countof(a) (sizeof(a)/sizeof((a)[0]))
#endif


#ifndef _ERRCODE_DEFINED
typedef int errno_t;
#endif


/**
* This method writes formatted data to a string.
*
* @param buffer A pointer to a wchar_t that is the storage location for output.
* @param sizeOfBuffer A size_t that indicates the maximum number of characters to store.
* @param format A const wchart_t that is the format control string.
* @param ... Optional parameters.
*
* @return An integer that indicates the number of characters written, or -1 is an error occurred.
*/
int swprintf_s( wchar_t* buffer, size_t sizeOfBuffer, const wchar_t* format, ... );

/**
* This method writes formatted data to a string.
*
* @param buffer A char pointer that is the storage location for output.
* @param sizeOfBuffer A size_t that indicates the maximum number of characters to store.
* @param format A const char pointer that is the format control string.
* @param ... Optional parameters.
*
* @return An integer that indicates the number of characters written, or -1 is an error occurred.
*/
int sprintf_s( char* buffer, size_t sizeOfBuffer, const char* format, ... );

/**
* This method converts a sequence of wide characters to a corresponding
* sequence of multibyte characters
*
* @param pReturnValue A pointer to a size_t that will get the number of characters converted.
* @param mbstr The address of a buffer for the resulting converted multibyte character string.
* @param sizeInBytes The size in bytes of the mbstr buffer.
* @param wcstr Points to the wide character string to be converted.
* @param count The maximum number of bytes to be stored in the mbstr buffer.
*
* @return An integer; Zero if successful, otherwise an error code.
*/
errno_t wcstombs_s(size_t *pReturnValue,char *mbstr, size_t sizeInBytes, const wchar_t *wcstr, size_t count);

/**
* This method converts a sequence of multibyte characters to a corresponding
* sequence of wide characters
*
* @param pReturnValue A pointer to a size_t that will get the number of characters converted.
* @param mbstr The address of a buffer for the resulting converted wide character string.
* @param sizeInWords The size of the wcstr buffer in words
* @param mbstr The address of a sequence of null terminated multibyte characters.
* @param count The maximum number of wide characters to store in the wcstr buffer, not including the terminating null.
*
* @return An integer; Zero if successful, otherwise an error code.
*/
errno_t mbstowcs_s(size_t *pReturnValue, wchar_t *wcstr, size_t sizeInWords, const char *mbstr, size_t count);

/**
* This method converts a string to uppercase.
*
* @param pcString A char pointer to the location of the string that needs to be converted.
*
* @return A char pointer to the location of the string that has been converted.
*/
char* strupr(char* pcString);

/**
* This method converts a string to lowercase.
*
* @param pcString A char pointer to the location of the string that needs to be converted.
*
* @return A char pointer to the location of the string that has been converted.
*/
char* strlwr(char* pcString);

/**
* This method returns a string in which the character order of a specified
* string is reversed.
*
* @param begin A char pointer to the beginning of the string to be reversed.
* @param end A char pointer to the end of the string to be reversed.
*/
void strreverse(char* begin, char* end);

/**
* This method returns a string in which the character order of a specified
* string is reversed.
*
* @param begin A wchar pointer to the beginning of the string to be reversed.
* @param end A wchar pointer to the end of the string to be reversed.
*/
void strreverseW(wchar_t* begin, wchar_t* end);

/**
* This method converts an integer to a wide string.
*
* @param value The number to be converted.
* @param str A wide char pointer where the result string will be stored.
* @param base An integer holding the base of value.
*/
void itow(int value, wchar_t* str, int base);

/**
* This method converts an integer to a string.
*
* @param value The number to be converted.
* @param buffer A char pointer where the result string will be stored.
* @param sizeInCharacters Size of buffer in single byte characters
* @param radix An integer holding the base of value.
*
* @return Zero if successful; an error code on failure
*/
errno_t _itoa_s( int value, char *buffer, size_t sizeInCharacters, int radix );

/**
* This method converts an integer to a wide string.
*
* @param value The number to be converted.
* @param buffer A wide char pointer where the result string will be stored.
* @param sizeInCharacters Size of buffer in wide characters
* @param radix An integer holding the base of value.
*
* @return Zero if successful; an error code on failure
*/
errno_t _itow_s( int value, wchar_t *buffer, size_t sizeInCharacters, int radix );

/**
* This method copies a string.
*
* @param a A char pointer to the location of the destination string buffer.
* @param size_t The size of the destination string buffer.
* @param b A const char pointer that is the null terminated source string.
*
* @return A char pointer to the destination string.
*/
char* strcpy_s(char* a, size_t sizeBuffer, const char* b);

/**
* This method copies a string.
*
* @param a A char pointer to the location of the destination string buffer.
* @param size_t The size of the destination string buffer.
* @param b A const char pointer that is the null terminated source string.
* @param c A size_t that indicates the number of characters to be copied.
*
* @return A char pointer to the destination string.
*/
char* strncpy_s(char* a, size_t sizeBuffer, const char* b, size_t c);

/**
* This method appends a string to another string.
*
* @param a A char pointer which is a null terminated destination string.
* @param size_t The size of the destination string buffer.
* @param b A const char pointer that is the null terminated source string.
*
* @return A char pointer to the destination string.
*/
char* strcat_s(char* a, size_t sizeBuffer, const char* b);

/**
* This method appends a string to another string.
*
* @param a A char pointer which is a null terminated destination string.
* @param size_t The size of the destination string buffer.
* @param b A const char pointer that is the null terminated source string.
* @param c A size_t that indicates the number of characters to be appended.
*
* @return A char pointer to the destination string.
*/
char* strncat_s(char* a, size_t sizeBuffer, const char* b, size_t c);

/**
* This method finds the next token in a string.
*
* @param a A char pointer that contains the token or tokens.
* @param b A const char pointer that contains the delimited characters.
* @param char** Used to store the position information between calls to strtok_s
*
* @return A char pointer to the next token found in the string.
*/
char* strtok_s(char* a, const char* b, char** c);

/**
* This method creates, modifies or deletes environment variables.
*
* @param a A char pointer holding the environment variable name.
* @param b A char pointer holding the value of the environment variable.
*
* @return An integer; Zero if successful otherwise an error code.
*/
int putenv_s(const char* a, const char* b);

/**
* This method converts a string to uppercase.
*
* @param a A char pointer to the location of the string that needs to be converted.
* @param size_t The size of the buffer that holds the string to be converted.
*
* @return A char pointer to the location of the string that has been converted.
*/
char* _strupr_s(char* a, size_t sizeBuffer);

/**
* This method copies characters between buffers.
*
* @param a A void pointer to the new buffer where characters will be copied.
* @param size_t The size of the new buffer.
* @param c A void pointer to the buffer from where the characters will be copied.
* @param d A size_t that indicates the number of characters to copy.
*/
void* memcpy_s(void* a, size_t sizeBuffer, void* c, size_t d);

/**
* This method opens a file.
*
* @param f A pointer to the file pointer that will receive the pointer to the opened file.
* @param fn A const char pointer to the filename.
* @param am A const char pointer to type of access permitted.
*
* @return An integer; Zero if successful, otherwise an error code.
*
* @notes If f, fn, or am is a null pointer, this function will generate an invalid parameter exception.
*/
int fopen_s(FILE** f, const char* fn, const char* am);

/**
* This method creates a temporary file.
*
* @param f The address of a pointer to store the address of the generated pointer to a stream.
*
* @return An integer; Zero if successful, otherwise an error code.
*/
int tmpfile_s(FILE** f);

/**
* This method converts a time value and corrects for the local time zone.
*
* @param pt A pointer to the time structure to be filled in.
* @param ptm A pointer to the stored time.
*
* @return An integer; Zero if successful, otherwise an error code.
*/
int localtime_s(struct tm* pt, const time_t* ptm);

/**
* This method converts a time value to a structure.
*
* @param pt A pointer to a tm structure. The fields of the returned structure hold the evaluated value of the timer argument in UTC rather than in local time.
* @param ptm A pointer to stored time. The time is represented as seconds elapsed since midnight (00:00:00), January 1, 1970, coordinated universal time (UTC).
*
* @return An integer; Zero if successful, otherwise an error code.
*/
int gmtime_s(struct tm* pt, const time_t* ptm);

#ifdef __cplusplus
}
#endif



#endif //#ifndef __GOT_SECURE_LIB__


