#include "CFD.h"
#include "SendHelper.h"
#include "FileUtils.h"
#include "DS100Parity.h"
#include <vector>
#include "Convert.h"
#include <iostream>
#include <chrono>

using namespace SendHelper;
using namespace std;

uint8_t CFD::reverse(uint8_t n)
{
	// Reverse the top and bottom nibble then swap them.
	return (lookupinv[n & 0x1111] << 4) | lookupinv[n >> 4];
}

// nMilliseconds = 0 means infinite
bool CFD::WaitForCTS(HANDLE hPort, size_t nMillisecondsTimeout)
{
	bool b = true;
	bool bReturn = true;
	DWORD Bytes = 0;
	unsigned char tCTS[4] = { '\0' };

	SendString(hPort, "AT\0333800\r"); // clear the latched register, latched register will capture <uS pulses

	auto begin = std::chrono::time_point_cast<std::chrono::milliseconds>(std::chrono::system_clock::now());
	
	while (b)
	{
		SendString(hPort, "AT\03309\r"); // read latched registers
		memset(tCTS, '\0', sizeof(tCTS));
		ReadFile(hPort, tCTS, 4, &Bytes, 0);
		if (Bytes != 3)
		{
			continue;
		}
		int number = (int)strtol((const char*)&tCTS[0], NULL, 16);
		if (number & 0x1)
		{
			printf("Parity error.\n");
			SendString(hPort, "AT\0338000\r");		// D - output off, F - output off, clear the latched register
			exit(0);
		}
		if (number & 0x2)
		{
			break;
		}

		auto end = std::chrono::time_point_cast<std::chrono::milliseconds>(std::chrono::system_clock::now());

		auto ms = std::chrono::duration_cast<std::chrono::milliseconds>(end - begin).count();
		if (nMillisecondsTimeout != 0 && ms >= nMillisecondsTimeout)
		{
			bReturn = false;
			break;
		}
	}
	SendString(hPort, "AT\0338000\r");		// D - output off, F - output off, clear the latched register
	return bReturn;
}

void CFD::SendKeyDS102KnownData(HANDLE hPort, bool bCTS, size_t nSendCount, size_t nTimeBetweenSends, size_t nWaitMillisecondsAfterCTS)
{
	bool b = true;
	unsigned char tStr[256] = { '\0' };
	unsigned char tCTS[4] = { '\0' };
	DWORD Bytes = 0;

	//// Setup DS102
	SendString(hPort, "AT\0333282\r");		// Baud Rate (3K)
	SendString(hPort, "AT\0333300\r");
	SendString(hPort, "AT\0330104\r");		// Set Mode DS102	
	SendString(hPort, "AT\0332300\r");		// Set CTS control to manual mode

	SendString(hPort, "AT\03322FF\r");		// blue
	SendString(hPort, "AT\0332000\r");		// red
	SendString(hPort, "AT\0332100\r");		// green

	std::wcout << L"Waiting for clear to send signal." << endl;
	WaitForCTS(hPort);
	Sleep(nWaitMillisecondsAfterCTS);

	tStr[0] = 0x8a;
	tStr[1] = 0xac;
	tStr[2] = 0xd5;
	tStr[3] = 0x38;
	tStr[4] = 0x01;
	tStr[5] = 0x54;
	tStr[6] = 0xa3;
	tStr[7] = 0xc6;
	tStr[8] = 0xae;
	tStr[9] = 0x40;
	tStr[10] = 0xb5;
	tStr[11] = 0x79;
	tStr[12] = 0xf8;
	tStr[13] = 0x23;
	tStr[14] = 0xd1;
	tStr[15] = 0x68;
	
	for (size_t i = 0; i < nSendCount; ++i)
	{ 
		WriteFile(hPort, tStr, 16, &Bytes, 0);
		if (i < nSendCount - 1)
		{
			Sleep(nTimeBetweenSends);
		}
	}
}

bool CFD::SendKeyDS102(HANDLE hPort, wstring strKeyFile, bool bCTS, std::wstring strTimer_TA, size_t nSendCount, size_t nTimeBetweenSends, size_t nWaitMillisecondsAfterCTS, std::wstring strCTSMessage, size_t nMillisecondsTimeout)
{
	bool bReturn = true;
	unsigned char tStr[256] = { '\0' };
	unsigned char tCTS[4] = { '\0' };
	DWORD Bytes = 0;
	
	size_t pos = strKeyFile.find_first_of(L"\\");
	if (wstring::npos == pos)
	{
		wstring path = CFileUtils::GetCurrentAppPath();
		path += L"\\";
		path += strKeyFile;
		strKeyFile = path;
	}

	size_t nTimer_tA = 225;

	if (strTimer_TA.length() > 0)
	{
		nTimer_tA = ::_wtoi(strTimer_TA.c_str());
	}

	vector<unsigned char> vecData;
	vecData = CFileUtils::FileRead(strKeyFile, 16, 65536);

	//// DS102
	SendString(hPort, "AT\0333282\r");		// Baud Rate (3K)
	SendString(hPort, "AT\0333300\r");
	SendString(hPort, "AT\0330104\r");		// Set Mode DS102	
	SendString(hPort, "AT\0332300\r");		// Set CTS control to manual mode

	SendString(hPort, "AT\03322FF\r");		// blue
	SendString(hPort, "AT\0332000\r");		// red
	SendString(hPort, "AT\0332100\r");		// green

	bReturn = WaitForCTS(hPort, nMillisecondsTimeout);
	if (false != bReturn)
	{
		Sleep(nWaitMillisecondsAfterCTS);
		for (size_t i = 0; i < nSendCount; ++i)
		{
			WriteFile(hPort, &vecData[0], vecData.size(), &Bytes, 0);
			if (i < nSendCount - 1)
			{
				Sleep(nTimeBetweenSends);
			}
		}
	}
	return bReturn;
}

bool CFD::SendKeyDS102Tempest(HANDLE hPort, wstring strKeyFile, size_t nSendCount, size_t nTimeBetweenSends, std::wstring strCTSMessage, size_t nMillisecondsTimeout)
{
	bool bReturn = true;
	unsigned char tStr[256] = { '\0' };
	unsigned char tCTS[4] = { '\0' };
	DWORD Bytes = 0;

	size_t pos = strKeyFile.find_first_of(L"\\");
	if (wstring::npos == pos)
	{
		wstring path = CFileUtils::GetCurrentAppPath();
		path += L"\\";
		path += strKeyFile;
		strKeyFile = path;
	}


	vector<unsigned char> vecData;
	vecData = CFileUtils::FileRead(strKeyFile, 1, 65536);

	//// DS102
	SendString(hPort, "AT\0333282\r");		// Baud Rate (3K)
	SendString(hPort, "AT\0333300\r");
	SendString(hPort, "AT\0330104\r");		// Set Mode DS102	
	SendString(hPort, "AT\0332300\r");		// Set CTS control to manual mode

	SendString(hPort, "AT\03322FF\r");		// blue
	SendString(hPort, "AT\0332000\r");		// red
	SendString(hPort, "AT\0332100\r");		// green


	for (size_t i = 0; i < nSendCount; ++i)
	{
		WriteFile(hPort, &vecData[0], vecData.size(), &Bytes, 0);
		if (i < nSendCount - 1)
		{
			Sleep(nTimeBetweenSends);
		}
	}

	return bReturn;
}

void CFD::SendKeyMode4(HANDLE hPort, std::wstring strKeyFileA, std::wstring strKeyFileB)
{
	bool b = true;
	unsigned char tStr[256] = { '\0' };
	unsigned char tCTS[4] = { '\0' };
	DWORD Bytes = 0;

	size_t posA = strKeyFileA.find_first_of(L"\\");
	if (wstring::npos == posA)
	{
		wstring path = CFileUtils::GetCurrentAppPath();
		path += L"\\";
		path += strKeyFileA;
		strKeyFileA = path;
	}
	size_t posB = strKeyFileB.find_first_of(L"\\");
	if (wstring::npos == posB)
	{
		wstring path = CFileUtils::GetCurrentAppPath();
		path += L"\\";
		path += strKeyFileB;
		strKeyFileB = path;
	}

	vector<unsigned char> vecDataA;
	vector<unsigned char> vecDataB;
	vecDataA = CFileUtils::FileRead(strKeyFileA, 16, 65536);
	vecDataB = CFileUtils::FileRead(strKeyFileB, 16, 65536);


	//// DS102
	SendString(hPort, "AT\0333282\r");		// Baud Rate (3K)
	SendString(hPort, "AT\0333335\r");
	SendString(hPort, "AT\0330104\r");		// Set Mode DS102	
	SendString(hPort, "AT\0332300\r");		// Set CTS control to manual mode

	SendString(hPort, "AT\03322FF\r");		// blue
	SendString(hPort, "AT\0332000\r");		// red
	SendString(hPort, "AT\0332100\r");		// green


	SendString(hPort, "AT\0330F01\r");		// Drop B
	Sleep(32);
	SendString(hPort, "AT\0331101\r");		// CTS ON
	Sleep(4);
	SendString(hPort, "AT\0331100\r");		// CTS OFF
	Sleep(80);
	SendString(hPort, "AT\0330F00\r");		// Raise B
	Sleep(40);
	SendString(hPort, "AT\0331101\r");		// CTS ON
	Sleep(200);

	WriteFile(hPort, &vecDataA[0], vecDataA.size(), &Bytes, 0);
	SendString(hPort, "AT\0331100\r");		// CTS OFF

	Sleep(1700);
	SendString(hPort, "AT\0330F01\r");		// Drop B
	Sleep(32);
	SendString(hPort, "AT\0331101\r");		// CTS ON
	Sleep(4);
	SendString(hPort, "AT\0331100\r");		// CTS OFF
	Sleep(80);
	SendString(hPort, "AT\0330F00\r");		// Raise B
	Sleep(40);
	SendString(hPort, "AT\0331101\r");		// CTS ON
	Sleep(200);

	WriteFile(hPort, &vecDataB[0], vecDataB.size(), &Bytes, 0);
	SendString(hPort, "AT\0331100\r");		// CTS OFF
}

void CFD::SendKeyMode4KnownData(HANDLE hPort)
{
	bool b = true;
	unsigned char tStr[256] = { '\0' };
	unsigned char tCTS[4] = { '\0' };
	DWORD Bytes = 0;

	//// DS102
	SendString(hPort, "AT\0333282\r");		// Baud Rate (3K)
	SendString(hPort, "AT\0333335\r");
	SendString(hPort, "AT\0330104\r");		// Set Mode DS102	
	SendString(hPort, "AT\0332300\r");		// Set CTS control to manual mode

	SendString(hPort, "AT\03322FF\r");		// blue
	SendString(hPort, "AT\0332000\r");		// red
	SendString(hPort, "AT\0332100\r");		// green


	SendString(hPort, "AT\0330F01\r");		// Drop B
	Sleep(32);
	SendString(hPort, "AT\0331101\r");		// CTS ON
	Sleep(4);
	SendString(hPort, "AT\0331100\r");		// CTS OFF
	Sleep(80);
	SendString(hPort, "AT\0330F00\r");		// Raise B
	Sleep(40);
	SendString(hPort, "AT\0331101\r");		// CTS ON
	Sleep(200);

	tStr[0] = 0x8a;
	tStr[1] = 0xac;
	tStr[2] = 0xd5;
	tStr[3] = 0x38;
	tStr[4] = 0x01;
	tStr[5] = 0x54;
	tStr[6] = 0xa3;
	tStr[7] = 0xc6;
	tStr[8] = 0xae;
	tStr[9] = 0x40;
	tStr[10] = 0xb5;
	tStr[11] = 0x79;
	tStr[12] = 0xf8;
	tStr[13] = 0x23;
	tStr[14] = 0xd1;
	tStr[15] = 0x68;


	WriteFile(hPort, tStr, 16, &Bytes, 0);
	SendString(hPort, "AT\0331100\r");		// CTS OFF

	Sleep(1700);
	SendString(hPort, "AT\0330F01\r");		// Drop B
	Sleep(32);
	SendString(hPort, "AT\0331101\r");		// CTS ON
	Sleep(4);
	SendString(hPort, "AT\0331100\r");		// CTS OFF
	Sleep(80);
	SendString(hPort, "AT\0330F00\r");		// Raise B
	Sleep(40);
	SendString(hPort, "AT\0331101\r");		// CTS ON
	Sleep(200);

	tStr[0] = 0x8a;
	tStr[1] = 0x6c;
	tStr[2] = 0xf8;
	tStr[3] = 0xbe;
	tStr[4] = 0xf4;
	tStr[5] = 0x70;
	tStr[6] = 0xf7;
	tStr[7] = 0x1b;
	tStr[8] = 0x4e;
	tStr[9] = 0x8a;
	tStr[10] = 0x56;
	tStr[11] = 0x0f;
	tStr[12] = 0x32;
	tStr[13] = 0xd9;
	tStr[14] = 0xc7;
	tStr[15] = 0x98;

	WriteFile(hPort, tStr, 16, &Bytes, 0);
	SendString(hPort, "AT\0331100\r");		// CTS OFF
}

void CFD::SendKeyDS102SincgarsMode2IcomKnownData(HANDLE hPort)
{
	bool b = true;
	unsigned char tStr[256] = { '\0' };
	unsigned char tCTS[4] = { '\0' };
	DWORD Bytes = 0;

	//// DS102
	SendString(hPort, "AT\0333230\r");		// Baud Rate (8K)
	SendString(hPort, "AT\03333D4\r");
	SendString(hPort, "AT\0330104\r");		// Set Mode DS102	
	SendString(hPort, "AT\0332300\r");		// Set CTS control to manual mode

	SendString(hPort, "AT\03322FF\r");		// blue
	SendString(hPort, "AT\0332000\r");		// red
	SendString(hPort, "AT\0332100\r");		// green

	SendString(hPort, "AT\0331801\r");		// F - output on
	SendString(hPort, "AT\0331700\r");		// F - F high - 0 volts


											// Data on D
	SendString(hPort, "AT\0331B00\r");		// Register 0x17   Register 0x1B   DS102DataOnBNotD   0 = Data on D, 1 = Data on B

											// change the data to B
	SendString(hPort, "AT\0331B01\r");		// Register 0x17   Register 0x1B   DS102DataOnBNotD   0 = Data on D, 1 = Data on B

											// turn on the output
	SendString(hPort, "AT\0331401\r");		// D - output on

	SendString(hPort, "AT\0331301\rAT\0331701\r");		// D - low and F low D=-5v, f=-5v

	Sleep(20);

	// there is a second at command here on purpose to give a very slight delay - it is required
	SendString(hPort, "AT\0331300\rAT\0331300\r\x02");		// D - high D=0, D - high, send 8 bits
															

	size_t n = 0;
	while (true)
	{
		ReadFile(hPort, tStr, 10, &Bytes, 0);
		if (Bytes > 0)
		{
			break;
		}
		n++;
	}
	printf("Number of bytes read %d\n", Bytes);


	SendString(hPort, "AT\0331B00\r");		// Register 0x17   Register 0x1B   DS102DataOnBNotD   0 = Data on D, 1 = Data on B

	printf("Start\n");

	printf("0x%c\n", tCTS[0]);


	unsigned char data[22][16] = {
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // TEK1 EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // TEK2 EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // TEK3 EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // TEK4 EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // TEK5 EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // TEK6 EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // ?  EMPTY
		{ 0x8C, 0x0D, 0x00, 0x00, 0x00, 0x19, 0x00, 0x00, 0x01, 0xD1, 0x87, 0x00, 0x02, 0xE1, 0x82, 0x80 }, // key
		{ 0xE9, 0x32, 0x67, 0xB0, 0x35, 0x96, 0x53, 0x74, 0x79, 0x4A, 0xEF, 0x88, 0xB5, 0x06, 0x83, 0x16 },
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // ? EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // ? EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // ? EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // ? EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // ? EMPTY
		{ 0x0E, 0x01, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00 }, // ? EMPTY
		{ 0x1E, 0x01, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x9E }, // ?
		{ 0x2E, 0x01, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x7F }, // ?
		{ 0x3E, 0x01, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xE1 }, // ?
		{ 0x4E, 0x01, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFE }, // ?
		{ 0x5E, 0x01, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x60 }, // ?
		{ 0x6E, 0x01, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x81 }, // ?
		{ 0x7E, 0x01, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x1F }  // ?
	};


	for (int i = 0; i < 23; i++)
	{
		// clear lacthed values
		// TDB
		// Latched bytes 000FEDCB
		SendString(hPort, "AT\0333800\r"); // clear the latched register, latched register will capture <uS pulses
		if (0 == i)
		{
			printf("Please press the Load button on the SINCGARS radio.\n");
		}

		while (true)
		{
			SendString(hPort, "AT\03309\r"); // read latched registers
			memset(tCTS, '\0', sizeof(tCTS));
			ReadFile(hPort, tCTS, 4, &Bytes, 0);
			printf("%d bytes read.", Bytes);
			
			if (Bytes != 3)
			{
				for (DWORD k = 0; k < Bytes; ++k)
				{
					printf("%0x", tCTS[k]);
				}
				printf("\n");
				continue;
			}

			int number = (int)strtol((const char*)&tCTS[0], NULL, 16);
			printf("The number is %d, bytes returned%d\n", number, Bytes);

			if (~number & 0x1)
			{
				printf("Parity error.\n");
				SendString(hPort, "AT\0331400\rAT\0331800\rAT\0338000\r");		// D - output off, F - output off, clear the latched register
				exit(0);
			}
			if (~number & 0x2)
			{
				break;
			}

		}

		printf("Loop %d\n", i);

		switch (i)
		{
			case 0:
				printf("Sleep 65\n");
				Sleep(65);
				break;
			case 22:
				printf("Last packet sent.\n");
				break;
			default:
				printf("Sleep 800\n");
				Sleep(800);
				break;
		}

		if (22 == i)
		{
			break;
		}

		printf("Sending slot %d\n", i);
		WriteFile(hPort, &data[i], 16, &Bytes, 0);
	}

	SendString(hPort, "AT\0331700\rAT\0338000\r");		// f=0v, clear the latched register

	SendString(hPort, "AT\0331400\rAT\0331800\rAT\0338000\r");		// D - output off, F - output off, clear the latched register
}

void CFD::SendKeyDS102SincgarsMode2Icom(HANDLE hPort, std::wstring strKeyFileBaseName, std::wstring strKeyFileExtension, std::wstring strTimer_t2A, std::wstring strTimer_t2B)
{
	int nCount = 1;
	bool b = true;
	unsigned char tStr[256] = { '\0' };
	unsigned char tCTS[4] = { '\0' };
	DWORD Bytes = 0;
	bool bLoopVar = true;
	wchar_t buf[MAX_PATH] = { L'\0' };

	//// DS102
	SendString(hPort, "AT\0333230\r");		// Baud Rate (8K)
	SendString(hPort, "AT\03333D4\r");
	SendString(hPort, "AT\0330104\r");		// Set Mode DS102	
	SendString(hPort, "AT\0332300\r");		// Set CTS control to manual mode

	SendString(hPort, "AT\03322FF\r");		// blue
	SendString(hPort, "AT\0332000\r");		// red
	SendString(hPort, "AT\0332100\r");		// green

	SendString(hPort, "AT\0331801\r");		// F - output on
	SendString(hPort, "AT\0331700\r");		// F - F high - 0 volts


											// Data on D
	SendString(hPort, "AT\0331B00\r");		// Register 0x17   Register 0x1B   DS102DataOnBNotD   0 = Data on D, 1 = Data on B

											// change the data to B
	SendString(hPort, "AT\0331B01\r");		// Register 0x17   Register 0x1B   DS102DataOnBNotD   0 = Data on D, 1 = Data on B

											// turn on the output
	SendString(hPort, "AT\0331401\r");		// D - output on

	SendString(hPort, "AT\0331301\rAT\0331701\r");		// D - low and F low D=-5v, f=-5v

	Sleep(20);

	// there is a second at command here on purpose to give a very slight delay - it is required
	SendString(hPort, "AT\0331300\rAT\0331300\r\x02");		// D - high D=0, D - high, send 8 bits


	SendString(hPort, "AT\0331B00\r");		// Register 0x17   Register 0x1B   DS102DataOnBNotD   0 = Data on D, 1 = Data on B

	vector< wstring > vecPath;
	while (bLoopVar)
	{
		swprintf_s(&buf[0], MAX_PATH, L"%s%d.%s", strKeyFileBaseName.c_str(), nCount, strKeyFileExtension.c_str());

		wstring path = CFileUtils::GetCurrentAppPath();
		path += L"\\";
		path += buf;


		bool bFileExists = CFileUtils::FileExists(path);
		if (bFileExists == false)
		{
			bLoopVar = false;
			break; // this will break out
		}

		size_t nFileSize = CFileUtils::FileGetSize(path);
		if (nFileSize == 0)
		{
			std::wcout << L"File " << path << L" contains no data. EOT." << endl;
		}
		else if (nFileSize < 16)
		{
			std::wcout << L"File " << path << L" is less than 16 bytes" << endl;
		}
		vecPath.push_back(path);
		nCount++;
	}


	// clear lacthed values
	// TDB
	// Latched bytes 000FEDCB
	SendString(hPort, "AT\0333800\r"); // clear the latched register, latched register will capture <uS pulses

	vector< wstring >::iterator it;
	size_t nCountFilesProc = 0;
	for (it = vecPath.begin(); it != vecPath.end(); ++it)
	{
		bool bReturn = true;
		DWORD Bytes = 0; \
		const size_t nLastKey = vecPath.size() - 1;

		size_t pos = it->find_first_of(L"\\");
		if (wstring::npos == pos)
		{
			wstring path = CFileUtils::GetCurrentAppPath();
			path += L"\\";
			path += *it;
			*it = path;
		}
		vector<unsigned char> vecData;
		vecData = CFileUtils::FileRead(*it, 16, 65536);

		if (0 == nCountFilesProc)
		{
			printf("Please press the Load button on the SINCGARS radio.\n");
		}
		WaitForCTS(hPort);

		if (0 == nCountFilesProc)
		{
			printf("Sleep 65\n");
			Sleep(65);
		}
		if (0 == nCountFilesProc && nLastKey != nCountFilesProc)
		{
			printf("Sleep 800\n");
			Sleep(800);
		}
		if (nLastKey == nCountFilesProc)
		{
			printf("Last packet being sent.\n");
		}

		printf("Sending slot %d\n", nCountFilesProc+1);

		WriteFile(hPort, &vecData[0], vecData.size(), &Bytes, 0);

		nCountFilesProc++;
	}

	SendString(hPort, "AT\0331700\rAT\0338000\r");		// f=0v, clear the latched register

	SendString(hPort, "AT\0331400\rAT\0331800\rAT\0338000\r");		// D - output off, F - output off, clear the latched register
}

void CFD::SendKeyDS102SincgarsMode1(HANDLE hPort, std::wstring strKeyFileBaseName, std::wstring strKeyFileExtension, std::wstring strTimer_t2A, std::wstring strTimer_t2B)
{
	int nCount = 1;
	bool bLoopVar = true;
	size_t nFileLoop = 0;
	wchar_t buf[MAX_PATH] = { L'\0' };
	size_t nTimer_t2A = 100;
	size_t nTimer_t2B = 100;

	if (strTimer_t2A.length() > 0)
	{
		nTimer_t2A = ::_wtoi(strTimer_t2A.c_str());
	}
	if (strTimer_t2B.length() > 0)
	{
		nTimer_t2B = ::_wtoi(strTimer_t2B.c_str());
	}	
	vector< wstring > vecPath;
	while (bLoopVar)
	{
		swprintf_s(&buf[0], MAX_PATH, L"%s%d.%s", strKeyFileBaseName.c_str(), nCount, strKeyFileExtension.c_str());

		wstring path = CFileUtils::GetCurrentAppPath();
		path += L"\\";
		path += buf;


		bool bFileExists = CFileUtils::FileExists(path);
		if (bFileExists == false)
		{
			bLoopVar = false;
			break; // this will break out
		}

		size_t nFileSize = CFileUtils::FileGetSize(path); 
		if (nFileSize == 0)
		{
			std::wcout << L"File " << path << L" contains no data. EOT." << endl;
		}
		else if (nFileSize < 16)
		{
			std::wcout << L"File " << path << L" is less than 16 bytes" << endl;
		}
		vecPath.push_back(path);
		nCount++;
	}

	//// DS102
	SendString(hPort, "AT\0333282\r");		// Baud Rate (3K)
	SendString(hPort, "AT\0333300\r");
	SendString(hPort, "AT\0330104\r");		// Set Mode DS102	
	SendString(hPort, "AT\0332300\r");		// Set CTS control to manual mode

	SendString(hPort, "AT\03322FF\r");		// blue
	SendString(hPort, "AT\0332000\r");		// red
	SendString(hPort, "AT\0332100\r");		// green


	vector< wstring >::iterator it;
	bool flag = false;
	bool flagKey1 = true;
	size_t nCountFilesProc = 0;
	for (it = vecPath.begin(); it != vecPath.end(); ++it)
	{
		bool bReturn = true;
		unsigned char tStr[256] = { '\0' };
		unsigned char tCTS[4] = { '\0' };
		DWORD Bytes = 0;
		nCountFilesProc++;

		size_t pos = it->find_first_of(L"\\");
		if (wstring::npos == pos)
		{
			wstring path = CFileUtils::GetCurrentAppPath();
			path += L"\\";
			path += *it;
			*it = path;
		}

		vector<unsigned char> vecData;
		vecData = CFileUtils::FileRead(*it, 16, 65536);

		if (false == flag)
		{
			wcout << L"Press LOAD button." << endl;
			WaitForCTS(hPort, 0);
			Sleep(nTimer_t2A);
			WriteFile(hPort, &vecData[0], vecData.size(), &Bytes, 0);
			flag = true;
			flagKey1 = false;
		}
		else
		{
			bool bResult = WaitForCTS(hPort, 1000);
			if (true == bResult)
			{
				Sleep(nTimer_t2B);
				WriteFile(hPort, &vecData[0], vecData.size(), &Bytes, 0);
			}
			else
			{
				if (false == flagKey1)
				{
					wcout << L"Press STO and location button." << endl;
					Sleep(10000);
				}
				if (nCountFilesProc < vecPath.size())
				{
					wcout << L"Press LOAD button." << endl;
					WaitForCTS(hPort, 0);
					Sleep(nTimer_t2A);
					WriteFile(hPort, &vecData[0], vecData.size(), &Bytes, 0);
					flag = true;
				}
			}
		}
	}
}


void CFD::SendKeyDS102KempleKnownData(HANDLE hPort, bool bCTS, size_t nSendCount, size_t nTimeBetweenSends)
{
	bool b = true;
	unsigned char tStr[256] = { '\0' };
	unsigned char tCTS[4] = { '\0' };
	unsigned char preamble = 0x07;
	unsigned char postamble = 0xFF;

	DWORD Bytes = 0;

	//// Setup DS102
	SendString(hPort, "AT\0333282\r");		// Baud Rate (3K)
	SendString(hPort, "AT\0333300\r");
	SendString(hPort, "AT\0330104\r");		// Set Mode DS102	
	SendString(hPort, "AT\0332300\r");		// Set CTS control to manual mode

	SendString(hPort, "AT\03322FF\r");		// blue
	SendString(hPort, "AT\0332000\r");		// red
	SendString(hPort, "AT\0332100\r");		// green

	if (true == bCTS)
	{
		while (b)
		{
			SendString(hPort, "AT\03302\r");
			memset(tCTS, '\0', 4);
			ReadFile(hPort, tCTS, 4, &Bytes, 0);
			if (tCTS[0] == '0' && tCTS[1] == '0' && tCTS[2] == '\r')
			{
				break;
			}
			Sleep(250);
		}
	}

	tStr[0] = 0x8a;
	tStr[1] = 0xac;
	tStr[2] = 0xd5;
	tStr[3] = 0x38;
	tStr[4] = 0x01;
	tStr[5] = 0x54;
	tStr[6] = 0xa3;
	tStr[7] = 0xc6;
	tStr[8] = 0xae;
	tStr[9] = 0x40;
	tStr[10] = 0xb5;
	tStr[11] = 0x79;
	tStr[12] = 0xf8;
	tStr[13] = 0x23;
	tStr[14] = 0xd1;
	tStr[15] = 0x68;
	tStr[16] = postamble;

	for (size_t i = 0; i < nSendCount; ++i)
	{
		WriteFile(hPort, &preamble, 1, &Bytes, 0);
		Sleep(95);
		WriteFile(hPort, &tStr[1], 17, &Bytes, 0);
		if (i < nSendCount - 1)
		{
			Sleep(nTimeBetweenSends);
		}
	}
}


void CFD::ReceiveTestKeysDS102CTS(HANDLE hPort, std::wstring strKeyFileBaseName, std::wstring strKeyFileExtension, size_t nTimeout)
{
	bool b = true;
	unsigned char tStr[256] = { '\0' };
	unsigned char tCTS[4] = { '\0' };
	DWORD Bytes = 0;
	vector<unsigned char> vecKey;
	size_t nKeyCount = 0;

	// **********************************	

	//// DS102
	SendString(hPort, "AT\0333282\r");		// Baud Rate (3K)
	SendString(hPort, "AT\0333335\r");
	SendString(hPort, "AT\0330104\r");		// Set Mode DS102	
	SendString(hPort, "AT\0332300\r");		// Set CTS control to manual mode

	SendString(hPort, "AT\03322FF\r");		// blue
	SendString(hPort, "AT\0332000\r");		// red
	SendString(hPort, "AT\0332100\r");		// green


	std::puts("Press Send on the SKL then enter any key followed by <Enter> to continue...");
	std::getchar();

	int  maxUntilDoneTime = 1000 * nTimeout; // 32 seconds is max - i just cannot wait
	int  totalWaitTime = 0;

	while (true)
	{
		memset(tStr, '\0', 256);
		vecKey.clear();

		// turn CTS ON - SKL ask to press LOAD
		// The minimum time is 1usec, typical is 52ms
		SendString(hPort, "AT\0331101\r");		// CTS ON
		Sleep(52);
		SendString(hPort, "AT\0331100\r");		// CTS OFF

												// give a little time to let data flow if possible - lets the buffer flow for a little before first read
		Sleep(80);

		int  sleepTime = 100;
		int  maxTimeUntilNextRequest = 2000;
		int  noDataCounter = 0;
		while (true)
		{
			ReadFile(hPort, tStr, 256, &Bytes, 0);
			if (Bytes > 0)
			{
				unsigned char* p = &tStr[0];
				printf("%d bytes received\n", Bytes);
				vecKey.insert(vecKey.end(), tStr, p + Bytes);
				totalWaitTime = 0;
				noDataCounter = 0;
				continue;
			}
			else
			{
				if (noDataCounter * sleepTime >= maxTimeUntilNextRequest)
				{
					// next request
					break;
				}
				Sleep(sleepTime);
				noDataCounter++;
			}
		}
		totalWaitTime += (noDataCounter * sleepTime);

		// print the keys
		for (auto a = vecKey.begin(); a < vecKey.end(); a++)
		{
			//char b = *a;
			unsigned char b = reverse(*a);
			printf("0x%02X, ", *a);
		}

		if (vecKey.size() > 0)
		{
			printf("\n");
			nKeyCount++;

			wstring fileName = strKeyFileBaseName;
			size_t posA = fileName.find_first_of(L"\\");
			if (wstring::npos == posA)
			{
				wstring path = CFileUtils::GetCurrentAppPath();
				path += L"\\";
				path += fileName;
				fileName = path;
			}
			fileName += CConvert::IntToStrW(nKeyCount);
			fileName += L".";
			fileName += strKeyFileExtension;
			CFileUtils::FileWrite(fileName, vecKey, true);
		}
		

		if (totalWaitTime >= maxUntilDoneTime)
		{
			break;
		}
	}
}


/*
void CFD::ReceiveDS102SincgarsMode2Icom(HANDLE hPort, std::wstring strKeyFileBaseName, std::wstring strKeyFileExtension, size_t nTimeout)
{
	bool b = true;
	unsigned char tStr[256] = { '\0' };
	unsigned char tCTS[4] = { '\0' };
	DWORD Bytes = 0;

	//// DS102
	SendString(hPort, "AT\0333230\r");		// Baud Rate (8K)
	SendString(hPort, "AT\03333D4\r");
	SendString(hPort, "AT\0330104\r");		// Set Mode DS102	
	SendString(hPort, "AT\0332300\r");		// Set CTS control to manual mode
											

	SendString(hPort, "AT\03322FF\r");		// blue
	SendString(hPort, "AT\0332000\r");		// red
	SendString(hPort, "AT\0332100\r");		// green

	SendString(hPort, "AT\0331801\r");		// F - output on
	SendString(hPort, "AT\0331700\r");		// F - F high - 0 volts


											// Data on D
	SendString(hPort, "AT\0331B00\r");		// Register 0x17   Register 0x1B   DS102DataOnBNotD   0 = Data on D, 1 = Data on B

											// change the data to B
	SendString(hPort, "AT\0331B01\r");		// Register 0x17   Register 0x1B   DS102DataOnBNotD   0 = Data on D, 1 = Data on B

											// turn on the output
	SendString(hPort, "AT\0331401\r");		// D - output on

	SendString(hPort, "AT\0331301\rAT\0331701\r");		// D - low and F low D=-5v, f=-5v

	Sleep(20);

	// there is a second at command here on purpose to give a very slight delay - it is required
	SendString(hPort, "AT\0331300\rAT\0331300\r\x02");		// D - high D=0


	size_t n = 0;
	while (true)
	{
		ReadFile(hPort, tStr, 1, &Bytes, 0);
		if (Bytes > 0)
		{
			break;
		}
		n++;
	}
	printf("Number of bytes read\n");


	SendString(hPort, "AT\0331B00\r");		// Register 0x17   Register 0x1B   DS102DataOnBNotD   0 = Data on D, 1 = Data on B

	SendString(hPort, "AT\03302\r");
	ReadFile(hPort, tCTS, 4, &Bytes, 0);

	printf("Start\n");

	printf("0x%c\n", tCTS[0]);


	unsigned char data[16][16] = {
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // TEK1 EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // TEK2 EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // TEK3 EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // TEK4 EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // TEK5 EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // TEK6 EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // ?  EMPTY
		{ 0x8C, 0x0D, 0x00, 0x00, 0x00, 0x19, 0x00, 0x00, 0x01, 0xD1, 0x87, 0x00, 0x02, 0xE1, 0x82, 0x80 }, // key
		{ 0xE9, 0x32, 0x67, 0xB0, 0x35, 0x96, 0x53, 0x74, 0x79, 0x4A, 0xEF, 0x88, 0xB5, 0x06, 0x83, 0x16 },
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // ? EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // ? EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // ? EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // ? EMPTY
		{ 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x7D, 0x77 }, // ? EMPTY
		{ 0x0E, 0x01, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x00 }, // ? EMPTY
		{ 0x1E, 0x01, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0xFF, 0x00, 0x9E } // ?
	};

	int i = 0;
	for (i = 0; i < 16; i++)
	{
		if (0 == i)
		{
			printf("Please press the Load button on the SINCGARS radio.\n");
		}

		while (true)
		{
			SendString(hPort, "AT\03302\r");
			ReadFile(hPort, tCTS, 4, &Bytes, 0);
			if (tCTS[0] == '0' && tCTS[1] == '0')
			{
				break;
			}
		}

		if (0 == i)
		{
			Sleep(65);
		}
		else
		{
			Sleep(800);
		}
		printf("Sending slot %d\n", i);
		WriteFile(hPort, &data[i], 16, &Bytes, 0);
		Sleep(1);
	}
}

*/