// DS102Test.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "StringUtils.h"
#include "ConfigAbs.h"
#include "ApexException.h"
#include "FileUtils.h"
#include "CFD.h"
#include <string>
#include <iostream>
#include <list>
#include <Windows.h>
#include <memory>

using namespace std;


DWORD SetupComPort(HANDLE& hPort, std::wstring strPort);
DWORD LogicSwitch(HANDLE& hPort, CConfigAbs& config);

int main()
{
	bool	bSend = false;
	int		nStatus = 0;
	wstring strAppPath;
	wstring strAppPathIni;


	try
	{
		strAppPath = CFileUtils::GetCurrentAppPath();
		strAppPathIni = strAppPath + _T("\\DS102Test.ini");

		wcout << L"DS102Test.exe started" << endl;
		wcout << L"Configuration file path is: " << strAppPathIni << endl;

		bool bExists = CFileUtils::FileExists(strAppPathIni);
		if (false == bExists)
		{
			wcout << L"Configuration file " << strAppPathIni << L" does not exists. Please create configuration file." << endl;
			exit(0);
		}


		CConfigAbs config;
		config.ReadConfig(strAppPathIni);

		wcout << L"Platformselected = " << config.GetValueW(L"Platform", L"Platform") << endl;
		wcout << L"    COM Port selected = " << config.GetValueW(config.GetValueW(L"Platform", L"Platform"), L"Port") << endl;
		wstring str = config.GetValueW(L"Platform", L"Platform");

		HANDLE hPort;
		DWORD nResult = SetupComPort(hPort, config.GetValueW(config.GetValueW(L"Platform", L"Platform"), L"Port"));
		if (ERROR_SUCCESS != nResult)
		{
			wcout << L"Error setting up COM port (" << nResult << L")" << endl;
			wcout << L"DS102Test.exe complete." << endl;
			exit(1);
		}
		wcout << L"COM port " << config.GetValueW(config.GetValueW(L"Platform", L"Platform"), L"Port") << L" opened." << endl;

		nResult = LogicSwitch(hPort, config);

	}
	catch (CApexException& ex)
	{
		std::wcout << ex.GetDescription() << std::endl;
	}

	wcout << L"DS102Test.exe complete." << endl;

    return 0;
}

wstring NormalizeComPortString(wstring strPort)
{
	//To specify a COM port number greater than 9, use the following syntax: "\\.\COM10". 
	wstring strPortTmp;
	wstring strPortLongName;

	size_t nPort = 0;
	for (size_t number = 0; number < strPort.length(); number++)
	{
		if (isdigit(strPort[number]))
		{
			strPortTmp += strPort[number];
		}
	}
	if (strPortTmp.length() < 1)
	{
		wstring strMsg = L"Error opening ";
		strMsg += strPort;
		strMsg += L". No port number found.";
		throw new CApexException(EnumSeverity::ERROR_FATAL, strMsg.c_str(), __FILE__, __LINE__);
	}
	else
	{
		nPort = _wtoi(strPortTmp.c_str());
		if (nPort <= 9)
		{
			strPortLongName = strPort;
		}
		else
		{
			strPortLongName = L"\\\\.\\COM";
			strPortLongName += strPortTmp;
		}
	}
	return strPortLongName;
}

DWORD SetupComPort(HANDLE& hPort, wstring strPort)
{
	DWORD err = 0;
	DCB				 PortDCB;
	DCB				 m_OriginalDCB;

	// change the com port connection string if the COM port if the port is > 9
	strPort = NormalizeComPortString(strPort);

	// Open the port.
	if ((hPort = CreateFile(
		strPort.c_str(), GENERIC_READ | GENERIC_WRITE, 0, NULL,
		OPEN_EXISTING, 0, NULL)) == INVALID_HANDLE_VALUE) {

		err = GetLastError();

		return err;
	}

	// Get the current device control settings.
	if (!GetCommState(hPort, &PortDCB)) {
		err = GetLastError();

		return(err);
	}
	// *** Configure the port.
	// Restore character transmission and place the transmission line in 
	// a nonbreak state.
	if (!EscapeCommFunction(hPort, CLRDTR/*BREAK*/)) {
		err = GetLastError();

		return(err);
	}

	// Get the current device control settings.
	if (!GetCommState(hPort, &PortDCB)) {
		err = GetLastError();

		return(err);
	}

	// Save the control settings so we restore later.
	m_OriginalDCB = PortDCB;

	// Set the new device control block settings.
	PortDCB.DCBlength = sizeof(DCB);

	// PCR #612 [DES] Baud rate must be 9600 for SKL
	PortDCB.BaudRate = 57600;
	PortDCB.fBinary = TRUE;
	PortDCB.fParity = TRUE;
	PortDCB.fOutxCtsFlow = FALSE;
	PortDCB.fOutxDsrFlow = FALSE;
	PortDCB.fDtrControl = DTR_CONTROL_ENABLE;
	PortDCB.fDsrSensitivity = FALSE;
	PortDCB.fTXContinueOnXoff = FALSE;
	PortDCB.fOutX = FALSE;
	PortDCB.fInX = FALSE;
	PortDCB.fErrorChar = FALSE;
	PortDCB.fNull = FALSE;
	PortDCB.fRtsControl = RTS_CONTROL_ENABLE;
	PortDCB.fAbortOnError = TRUE;
	PortDCB.ByteSize = 8;
	PortDCB.Parity = NOPARITY;
	PortDCB.StopBits = ONESTOPBIT;

	// Init port with new settings.
	if (!SetCommState(hPort, &PortDCB)) {
		err = GetLastError();

		return(err);
	}

	if (!EscapeCommFunction(hPort, CLRBREAK)) {
		err = GetLastError();

		return(err);
	}

	// set short timeouts on the comm port.
	COMMTIMEOUTS timeouts;

	timeouts.ReadIntervalTimeout = 1;
	timeouts.ReadTotalTimeoutMultiplier = 1;
	timeouts.ReadTotalTimeoutConstant = 1;
	timeouts.WriteTotalTimeoutMultiplier = 1;
	timeouts.WriteTotalTimeoutConstant = 1;
	SetCommTimeouts(hPort, &timeouts);


	return ERROR_SUCCESS;
}

DWORD LogicSwitch(HANDLE& hPort, CConfigAbs& config)
{
	wstring strPlatform = CStringUtils::ToUpper(config.GetValueW(L"Platform", L"Platform"));

	if (L"PRIMARY_CFD" == strPlatform)
	{
		wcout << "PRIMARY_CFD" << endl; 
		CFD::SendKeyDS102(
			hPort, 
			config.GetValueW(strPlatform, L"BinaryKeyFile"), 
			false, 
			config.GetValueW(strPlatform, L"strTimer_TA"), 
			_wtoi(config.GetValueW(strPlatform, L"SendCount").c_str()),
			_wtoi(config.GetValueW(strPlatform, L"TimeBetweenSend").c_str()),
			_wtoi(config.GetValueW(strPlatform, L"WaitMillisecondsAfterCTS").c_str()));
	}
	else if (L"PRIMARY_CFD_CTS" == strPlatform)
	{
		wcout << "PRIMARY_CFD_CTS" << endl;
		CFD::SendKeyDS102(hPort, config.GetValueW(strPlatform, L"BinaryKeyFile"),
			true,
			config.GetValueW(strPlatform, L"strTimer_TA"),
			_wtoi(config.GetValueW(strPlatform, L"SendCount").c_str()),
			_wtoi(config.GetValueW(strPlatform, L"TimeBetweenSend").c_str()),
			_wtoi(config.GetValueW(strPlatform, L"WaitMillisecondsAfterCTS").c_str()));
	}
	else if (L"PRIMARY_CFD_KNOWNDATA" == strPlatform)
	{
		wcout << "PRIMARY_CFD_KNOWNDATA selected" << endl;
		CFD::SendKeyDS102KnownData(hPort, 
			false,
			_wtoi(config.GetValueW(strPlatform, L"SendCount").c_str()),
			_wtoi(config.GetValueW(strPlatform, L"TimeBetweenSend").c_str()),
			_wtoi(config.GetValueW(strPlatform, L"WaitMillisecondsAfterCTS").c_str()));
	}
	else if (L"PRIMARY_CFD_KNOWNDATA_CTS" == strPlatform)
	{
		wcout << "PRIMARY_CFD_KNOWNDATA_CTS selected" << endl;
		CFD::SendKeyDS102KnownData(hPort,
			true,
			_wtoi(config.GetValueW(strPlatform, L"SendCount").c_str()),
			_wtoi(config.GetValueW(strPlatform, L"TimeBetweenSend").c_str()),
			_wtoi(config.GetValueW(strPlatform, L"WaitMillisecondsAfterCTS").c_str()));
	}
	else if (L"PRIMARY_KOI18" == strPlatform)
	{
		wcout << "PRIMARY_KOI18" << endl;
		CFD::SendKeyDS102(
			hPort, 
			config.GetValueW(strPlatform, L"BinaryKeyFile"), 
			false, 
			config.GetValueW(strPlatform, L"strTimer_TA"),
			1,
			4000,
			_wtoi(config.GetValueW(strPlatform, L"WaitMillisecondsAfterCTS").c_str()));
	}
	else if (L"PRIMARY_TEMPEST" == strPlatform)
	{
		wcout << "PRIMARY_TEMPEST" << endl;
		CFD::SendKeyDS102Tempest(
			hPort,
			config.GetValueW(strPlatform, L"BinaryKeyFile"),
			_wtoi(config.GetValueW(strPlatform, L"SendCount").c_str()),
			_wtoi(config.GetValueW(strPlatform, L"TimeBetweenSend").c_str()));
	}
	else if (L"PRIMARY_MODE4" == strPlatform)
	{
		wcout << "PRIMARY_MODE4" << endl;
		CFD::SendKeyMode4(hPort,
			config.GetValueW(strPlatform, L"BinaryKeyFileA"),
			config.GetValueW(strPlatform, L"BinaryKeyFileB"));
	}
	else if (L"PRIMARY_MODE4_KNOWNDATA" == strPlatform)
	{
		wcout << "PRIMARY_MODE4_KNOWNDATA" << endl;
		CFD::SendKeyMode4KnownData(hPort);
	}
	else if (L"SECONDARY_DS102_CTS" == strPlatform  || L"SECONDARY_SINCGARS_MODE1" == strPlatform)
	{
		wcout << "SECONDARY_DS102_CTS" << endl;
		CFD::ReceiveTestKeysDS102CTS(hPort,
			config.GetValueW(strPlatform, L"BinaryKeyFileBaseName"),
			config.GetValueW(strPlatform, L"BinaryKeyFileExtension"),
			_wtoi(config.GetValueW(strPlatform, L"Timeout").c_str()));
	}
	else if (L"PRIMARY_SINCGARS_MODE2_ICOM_KNOWNDATA" == strPlatform)
	{
		wcout << L"PRIMARY_SINCGARS_MODE2_ICOM_KNOWNDATA" << endl;
		CFD::SendKeyDS102SincgarsMode2IcomKnownData(hPort);
	}
	else if (L"PRIMARY_SINCGARS_MODE2_ICOM" == strPlatform)
	{
		wcout << L"PRIMARY_SINCGARS_MODE2_ICOM" << endl;
		CFD::SendKeyDS102SincgarsMode2Icom(hPort,
			config.GetValueW(strPlatform, L"BinaryKeyFileBaseName"),
			config.GetValueW(strPlatform, L"BinaryKeyFileExtension"),
			config.GetValueW(strPlatform, L"Timer_t2A"),
			config.GetValueW(strPlatform, L"Timer_t2B"));
	}
	else if (L"PRIMARY_SINCGARS_MODE1" == strPlatform)
	{
		wcout << L"PRIMARY_SINCGARS_MODE1" << endl;
		CFD::SendKeyDS102SincgarsMode1(hPort,
			config.GetValueW(strPlatform, L"BinaryKeyFileBaseName"),
			config.GetValueW(strPlatform, L"BinaryKeyFileExtension"),
			config.GetValueW(strPlatform, L"Timer_t2A"),
			config.GetValueW(strPlatform, L"Timer_t2B")
			);
	}
	else
	{
		wcout << "Unknown platform " << strPlatform << L"." << endl;
		return ERROR_INVALID_DATA;
	}

	return ERROR_SUCCESS;
}