/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file ConfigFileImp.h
 * Header file for the libconfig concrete implementation class that
 * defines methods for reading and writing configuration information
 * from a file.
 */


//////////////////////////////////////////////////////////////////////
// pragmas
//////////////////////////////////////////////////////////////////////

#pragma once


//////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////

#include "ConfigImp.h"



/**
 * This is the concrete implementation class for gathering configuration
 * information from a file. The format of the file is expected to be
 * <Section>
 * <key> = <value> - with potentially multiple key/value pairs per section
 *
 * @author  Dave Webster
 */
class CConfigFileImp  : public CConfigImp
{
public:

	/**
	 * Save the information in the table out to a file
	 * 
	 * @param strFilename A std::string containing the file name to save the file.
	 * 
	 * @return:	0 for success and >0 for failure
	 *
	 * @notes This function has only real significance to config file parsing.
	 *        It saves the configuration file output after normalization
	 *        to another file.
	 */
	unsigned long SaveConfig(std::wstring filename);

	
	/** 
	 * Parse the configuration source and store the sections/subkeys
	 * and key-value/value name-value data pairs into a hash table 
	 * map.
	 * 
	 * @param strFilename The file name std::string or the root key 
	 *                    std::string to parse
	 * 
	 * @return	0 for success and 1 for error
	 */
	unsigned long ReadConfig(std::wstring filename);


	/**
	 * Constructor
	 */
	CConfigFileImp();


	/**
	 * Destructor
	 */
	virtual ~CConfigFileImp();
};

