/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file ConfigRegImp.h
 * Header file for the libconfig concrete implementation that defines
 * methods for reading and writing configuration information from
 * the windows registry.
 */


//////////////////////////////////////////////////////////////////////
// pragmas
//////////////////////////////////////////////////////////////////////

#pragma once


//////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////

#include "ConfigImp.h"


/**
 * This is the concrete implementation class for gathering configuration
 * information from the windows registry. The format of the registry is
 * expected to be
 * <Subkey>
 * <data name> = <data value> - with potentially multiple name/value pairs
 *								per subkey.
 * Keys under a subkey results in an error, the root key to search for is
 * expected to have subkeys only one level deep.
 *
 * 
 * @todo Add saving of values to the registry
 */
class CConfigRegImp : public CConfigImp
{
public:

	/**
	 * Constructor
	 */
	CConfigRegImp();

	
	/**
	 * Destructor
	 */
	virtual ~CConfigRegImp();


	/** 
	 * Parse the configuration source and store the sections/subkeys
	 * and key-value/value name-value data pairs into a hash table 
	 * map.
	 * 
	 * @param strFilename The file name wide string or the root key 
	 *                    wide string to parse
	 * 
	 * @return	0 for success and >0 for error
	 */
	unsigned long ReadConfig( std::wstring szhKey);

};
